uniform sampler2D userSources;
uniform sampler2D fields;
uniform sampler2D curls;
uniform sampler2D normCurlGrads;
uniform float dt;
uniform float dx;
uniform float dy;

bool isBorderPixel(vec2 texCoord) {
   return texCoord.s < dx || texCoord.t < dy || (1.0 - texCoord.s) < dx ||
      (1.0 - texCoord.t) < dy;
}

void main() {
   if (isBorderPixel(gl_TexCoord[0].st)) {
      gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
      return;
   }
   
   vec4 userSource = texture2D(userSources, gl_TexCoord[0].st);
   
   const float BUOY_COEFF = 0.002;
   vec4 field = texture2D(fields, gl_TexCoord[0].st);
   vec4 buoy = vec4(0.0, BUOY_COEFF * field.a, 0.0, 0.0);
   
   float w = texture2D(curls, gl_TexCoord[0].st).a;
   vec2 normCurlGrad = texture2D(normCurlGrads, gl_TexCoord[0].st).xy;
   
   const float EPSILON = 0.04;
   vec2 vorticity = w * EPSILON * vec2(normCurlGrad.y, -normCurlGrad.x);
   
   vec4 force = userSource + dt * (buoy + vec4(vorticity, 0.0, 0.0));
     
   gl_FragColor = force + field;
}
